from datetime import (
    datetime,
    timedelta,
)
from functools import partial

import pytest

from greg.models import (
    OrganizationalUnit,
    Person,
    PersonIdentity,
    PersonRole,
    Role,
    Sponsor,
)

person_role_with = partial(
    PersonRole.objects.create,
    start_date="2020-03-05",
    end_date="2020-06-10",
    contact_person_unit="Contact Person",
    available_in_search=True,
)


@pytest.fixture
def role_foo() -> Role:
    return Role.objects.create(type="role_foo", name_en="Role Foo")


@pytest.fixture
def role_bar() -> Role:
    return Role.objects.create(type="role_bar", name_en="Role Bar")


@pytest.fixture
def person(role_foo: Role, role_bar: Role) -> Person:
    person = Person.objects.create(
        first_name="Test",
        last_name="Tester",
        date_of_birth="2000-01-27",
        email="test@example.org",
        mobile_phone="123456789",
    )

    ou = OrganizationalUnit.objects.create(orgreg_id="12345", name_en="Test unit")
    person_role_with(
        person=person,
        role=role_foo,
        unit=ou,
        registered_by=Sponsor.objects.create(feide_id="foosponsor@uio.no"),
    )
    person_role_with(
        person=person,
        role=role_bar,
        unit=ou,
        registered_by=Sponsor.objects.create(feide_id="barsponsor@uio.no"),
    )

    return person


@pytest.fixture
def a_year_ago() -> datetime:
    return datetime.now() - timedelta(days=365)


@pytest.fixture
def feide_id(a_year_ago: datetime) -> PersonIdentity:
    return PersonIdentity(
        type=PersonIdentity.IdentityType.FEIDE_ID,
        verified_when=a_year_ago,
    )


@pytest.fixture
def passport(a_year_ago: datetime) -> PersonIdentity:
    return PersonIdentity(
        type=PersonIdentity.IdentityType.PASSPORT,
        verified_when=a_year_ago,
    )


@pytest.fixture
def feide_verified(person: Person, feide_id: PersonIdentity) -> Person:
    person.identities.add(feide_id, bulk=False)
    return person


@pytest.mark.django_db
def test_add_multiple_roles_to_person(person, role_foo, role_bar):
    person_roles = person.roles.all()
    assert len(person_roles) == 2
    assert role_foo in person_roles
    assert role_bar in person_roles


@pytest.mark.django_db
def test_identities(person: Person, feide_id: PersonIdentity, passport: PersonIdentity):
    person.identities.add(feide_id, bulk=False)
    assert list(person.identities.all()) == [feide_id]
    person.identities.add(passport, bulk=False)
    assert list(person.identities.all()) == [feide_id, passport]
